C*******************************************************************************

      SUBROUTINE  add_node (caller, callee)

C*******************************************************************************
C
C      Subroutine ADD_NODE stores an X-Calls-Y relationship in the call
C      graph.  A node for Y is added to X's CALLS list and a node for X
C      is added to Y's CALLED_BY list.
C
C
C      Arguments:
C
C          CALLER  (Integer)
C              is the index in the routine table of the calling routine.
C          CALLEE  (Integer)
C              is the index in the routine table of the called routine.
C
C*******************************************************************************

      IMPLICIT  NONE

C...      Parameters, global variables, and external definitions.

      INCLUDE  'struchart.inc'                  ! STRUCHART parameters and data structures.

C...      Subroutine arguments.

      INTEGER*4  caller, callee

C...      Local variables.

      INTEGER*4  current, index, new, previous
      LOGICAL  found





C...      Search the called routine in the calling routine's list of
C      called subroutines.

      current = calls(caller)
      previous = NULL
      found = .FALSE.
      DO WHILE ((.NOT. found) .AND. (current .NE. NULL))
          IF (adj_info(current) .EQ. callee) THEN
              found = .TRUE.
          ELSE
              previous = current
              current = adj_link(current)
          ENDIF
      ENDDO


C...      Add the called routine to the calling routine's list of called
C      subroutines.

      IF (.NOT. found) THEN
          IF (num_adj .GE. MAX_NUM_NODES) THEN
              WRITE (6,*)  '** Number of nodes in call graph exceeds ',
     +                     MAX_NUM_NODES, '. **'
              CALL EXIT
          ENDIF
          num_adj = num_adj + 1
          new = num_adj
          adj_info(new) = callee
          adj_link(new) = NULL
          IF (previous .NE. NULL) THEN     ! Add to the end of the list.
              adj_link(previous) = new
          ELSE                             ! Create new list.
              calls(caller) = new
          ENDIF
      ENDIF


C...      Search the calling routine in the called routine's sorted list
C      of calling subroutines.

      current = called_by(callee)
      previous = NULL
      found = .FALSE.
      DO WHILE ((.NOT. found) .AND. (current .NE. NULL))
          index = adj_info(current)
          IF (routine_name(caller) .LE. routine_name(index)) THEN
              found = .TRUE.
          ELSE
              previous = current
              current = adj_link(current)
          ENDIF
      ENDDO

      IF (found .AND. (routine_name(caller) .NE. (routine_name(index))))
     +    found = .FALSE.


C...      Add the calling routine to the called routine's sorted list of
C      calling subroutines and increment the called routine's in-degree.

      IF (.NOT. found) THEN
          IF (num_adj .GE. MAX_NUM_NODES) THEN
              WRITE (6,*)  '** Number of nodes in call graph exceeds ',
     +                     MAX_NUM_NODES, '. **'
              CALL EXIT
          ENDIF
          num_adj = num_adj + 1
          new = num_adj
          adj_info(new) = caller
          IF (previous .NE. NULL) THEN     ! Add to the middle or end of the list.
              adj_link(new) = adj_link(previous)
              adj_link(previous) = new
          ELSE                             ! Add to the beginning of the list.
              adj_link(new) = called_by(callee)
              called_by(callee) = new
          ENDIF
          in_degree(callee) = in_degree(callee) + 1
      ENDIF


      RETURN


      END
