C*******************************************************************************

      INTEGER*4  FUNCTION  fparse (file_spec, default_spec,
     +                             related_spec, field, parse_type,
     +                             result)

C*******************************************************************************
C
C      Function FPARSE emulates the DCL lexical function, "F$PARSE",
C      using the RMS SYS$PARSE service.  Given a file specification,
C      FPARSE will expand the file specification, translating logical
C      names and filling in defaults for missing fields.  The fully-
C      expanded file specification, or parts of it if specified, are
C      returned to the calling routine.  See the section on lexical
C      functions in the VAX/VMS DCL Dictionary for more information
C      about "F$PARSE".
C
C      NOTE that although the input arguments are declared as records
C      of type descriptor, you should pass them in as CHARACTER strings.
C      This was necessary to allow FPARSE to detect null arguments.
C      The descriptor type chosen, "DSCDEF1", was as close as I
C      could get to a character string descriptor type.  Specify
C      "%DESCR (...)" explicitly when passing string literals to
C      FPARSE; while attempting to maintain backwards compatibility
C      with older FORTRAN code, the VMS Linker stumbles over string
C      literals in this case.
C
C
C      Arguments (see the NOTE above about arguments of type CHARACTER):
C
C          FILE_SPEC  (Character)
C
C              The primary file specification being parsed.  This
C              argument may be null (unlike the DCL function where
C              it is required); if not specified, the default and
C              related file specifications are applied as usual to
C              a blank, primary file name.
C
C          DEFAULT_SPEC  (Character)
C
C              The default file specification.  If the default file
C              specification is specified (this argument may be
C              null), it provides substitutes for missing components
C              in the expanded, primary file specification.
C
C          RELATED_SPEC  (Character)
C
C              The related file specification.  If the related file
C              specification is specified (this argument may be
C              null), it provides substitutes for missing components
C              in the expanded, primary/default file specification.
C
C          FIELD  (Character)
C
C              Designates the field in the expanded file
C              specification that you want returned to you:
C
C                  "NODE"
C                  "DEVICE"
C                  "DIRECTORY"
C                  "NAME"
C                  "TYPE"
C                  "VERSION"
C
C              If this argument is null (or is an invalid field
C              name), the entire, fully-expanded file specification
C              is returned to the calling routine.
C
C          PARSE_TYPE  (Character)
C
C              Specifies the type of parse to perform.  There are
C              two options:
C
C                  "NO_CONCEAL" reveals concealed logical names.
C
C                  "SYNTAX_ONLY" does not verify that the device
C                      and directory in the fully-expanded file
C                      specification actually exist.
C
C          RESULT  (Character)
C
C              Returns the parsed file specification.  If a desired
C              field is not specified (null FIELD argument), the
C              entire, fully-expanded file specification is returned
C              to the calling routine.  Otherwise, only the specified
C              field is returned to the calling routine:
C
C                  "NODE"       returns "<node>::" (usually null)
C                  "DEVICE"     returns "<disk>:"
C                  "DIRECTORY"  returns "[<directory>]"
C                  "NAME"       returns "<file name>"
C                  "TYPE"       returns ".<extension>"
C                  "VERSION"    returns ";<number>" (";" if no version number)
C
C          FPARSE  (Function Value)
C
C              Returns the VAX/VMS status code returned by SYS$PARSE.
C              There are actually two calls to SYS$PARSE, one to
C              parse the related file specification and a second to
C              parse the primary file specification.  An error in
C              either parse operation is immediately returned to
C              the calling routine.  A disk or directory not found
C              error could be returned if you didn't specify
C              "SYNTAX_ONLY".  RMS$_NORMAL is returned if there
C              are no errors.
C
C*******************************************************************************

      IMPLICIT  NONE

C...      Parameters and external definitions.

      INCLUDE '($DSCDEF)'               ! Descriptor type definitions.
      INCLUDE '($FABDEF)'               ! RMS File Access Block (FAB) definitions.
      INCLUDE '($NAMDEF)'               ! RMS name block field definitions.
      INCLUDE '($SYSSRVNAM)'            ! VMS system service entry point descriptions.

C...      Subroutine arguments.

      RECORD /DSCDEF1/  file_spec       ! Character string.
      RECORD /DSCDEF1/  default_spec    ! Character string.
      RECORD /DSCDEF1/  related_spec    ! Character string.
      RECORD /DSCDEF1/  field           ! Character string.
      RECORD /DSCDEF1/  parse_type      ! Character string.
      CHARACTER*(*)  result

C...      Local variables.

      CHARACTER*16  field_name, type_of_parse
      CHARACTER*(NAM$C_MAXRSS)  expanded_string, rlf_expanded_string
      INTEGER*4  length, nop, status

      RECORD  /FABDEF/  fab             ! RMS file access block.
      RECORD  /NAMDEF/  nam             ! RMS name block.
      RECORD  /NAMDEF/  rlf             ! RMS related file name block.





      result = ' '              ! Return blank result in case of error.


C...      Copy the field name and parse type strings into local
C      character strings.

      IF (%LOC (field) .EQ. 0) THEN
          field_name = ' '
      ELSE
          CALL LIB$MOVC5 (field.DSC$W_MAXSTRLEN,
     +                    %VAL(field.DSC$A_POINTER),
     +                    %REF(' '),
     +                    LEN(field_name), %REF(field_name))
      ENDIF

      IF (%LOC (parse_type) .EQ. 0) THEN
          type_of_parse = ' '
      ELSE
          CALL LIB$MOVC5 (parse_type.DSC$W_MAXSTRLEN,
     +                    %VAL(parse_type.DSC$A_POINTER),
     +                    %REF(' '),
     +                    LEN(type_of_parse), %REF(type_of_parse))
      ENDIF


C...      Parse the related file specification.  Do not expand concealed
C      logical names and don't verify that its directory exists.

      IF (%LOC (related_spec) .NE. 0) THEN

          CALL LIB$MOVC5 (0, 0, 0, FAB$C_BLN, fab)      ! Zero all the fields.
          fab.FAB$B_BID = FAB$C_BID                     ! Block identifier.
          fab.FAB$B_BLN = FAB$C_BLN                     ! Block length.
          fab.FAB$L_FOP = FAB$M_NAM                     ! File-processing options.
          fab.FAB$L_NAM = %LOC (rlf)                    ! Name block (address).
          fab.FAB$L_FNA = related_spec.DSC$A_POINTER    ! File name.
          CALL LIB$MOVC3 (1, related_spec.DSC$W_MAXSTRLEN,
     +                    fab.FAB$B_FNS)

          CALL LIB$MOVC5 (0, 0, 0, NAM$C_BLN, rlf)      ! Zero all the fields.
          rlf.NAM$B_BID = NAM$C_BID                     ! Block identifier.
          rlf.NAM$B_BLN = NAM$C_BLN                     ! Block length.
          rlf.NAM$B_NOP = NAM$M_SYNCHK                  ! Name block options.
          rlf.NAM$L_ESA = %LOC (rlf_expanded_string)    ! Expanded string.
          rlf.NAM$B_ESS = NAM$C_MAXRSS

          status = SYS$PARSE (fab)

          IF (.NOT. status) THEN
              fparse = status
              RETURN
          ENDIF

          rlf.NAM$L_RSA = rlf.NAM$L_ESA                 ! Resultant string.
          rlf.NAM$B_RSL = rlf.NAM$B_ESL

      ENDIF


C...      Parse the main file specification, applying the default and
C      related file specifications.

      CALL LIB$MOVC5 (0, 0, 0, FAB$C_BLN, fab)          ! Zero all the fields.
      fab.FAB$B_BID = FAB$C_BID                         ! Block identifier.
      fab.FAB$B_BLN = FAB$C_BLN                         ! Block length.
      fab.FAB$L_FOP = FAB$M_NAM                         ! File-processing options.
      fab.FAB$L_NAM = %LOC (nam)                        ! Name block (address).
      IF (%LOC (file_spec) .NE. 0) THEN
          fab.FAB$L_FNA = file_spec.dsc$a_pointer       ! File name.
          CALL LIB$MOVC3 (1, file_spec.DSC$W_MAXSTRLEN, fab.FAB$B_FNS)
      ENDIF
      IF (%LOC (default_spec) .NE. 0) THEN
          fab.FAB$L_DNA = default_spec.DSC$A_POINTER    ! Default file specification.
          CALL LIB$MOVC3 (1, default_spec.DSC$W_MAXSTRLEN,
     +                    fab.FAB$B_DNS)
      ENDIF

      CALL LIB$MOVC5 (0, 0, 0, NAM$C_BLN, nam)          ! Zero all the fields.
      nam.NAM$B_BID = NAM$C_BID                         ! Block identifier.
      nam.NAM$B_BLN = NAM$C_BLN                         ! Block length.
      IF (type_of_parse .EQ. 'NO_CONCEAL') THEN
          nam.NAM$B_NOP = NAM$M_NOCONCEAL               ! Name block option (expand concealed logical names).
      ELSEIF (type_of_parse .EQ. 'SYNTAX_ONLY') THEN
          nam.NAM$B_NOP = NAM$M_SYNCHK                  ! Name block option (don't verify that directory exists).
      ENDIF
      IF (%LOC (related_spec) .NE. 0)                   ! Related file name block (address).
     +    nam.NAM$L_RLF = %LOC (rlf)
      nam.NAM$L_ESA = %LOC (expanded_string)            ! Expanded string.
      nam.NAM$B_ESS = NAM$C_MAXRSS

      status = SYS$PARSE (fab)

      IF (.NOT. status) THEN
          fparse = status
          RETURN
      ENDIF


C...      Return the desired field.  The completely expanded file
C      specification is stored by RMS in EXPANDED_STRING; the lengths
C      and addresses of the individual fields are available in the name
C      block.

      IF (field_name .EQ. 'NODE') THEN

          length = ZEXT (nam.NAM$B_NODE)
          CALL LIB$MOVC5 (length, %VAL(nam.NAM$L_NODE), %REF(' '),
     +                    LEN(result), %REF(result))

      ELSEIF (field_name .EQ. 'DEVICE') THEN

          length = ZEXT (nam.NAM$B_DEV)
          CALL LIB$MOVC5 (length, %VAL(nam.NAM$L_DEV), %REF(' '),
     +                    LEN(result), %REF(result))

      ELSEIF (field_name .EQ. 'DIRECTORY') THEN

          length = ZEXT (nam.NAM$B_DIR)
          CALL LIB$MOVC5 (length, %VAL(nam.NAM$L_DIR), %REF(' '),
     +                    LEN(result), %REF(result))

      ELSEIF (field_name .EQ. 'NAME') THEN

          length = ZEXT (nam.NAM$B_NAME)
          CALL LIB$MOVC5 (length, %VAL(nam.NAM$L_NAME), %REF(' '),
     +                    LEN(result), %REF(result))

      ELSEIF (field_name .EQ. 'TYPE') THEN

          length = ZEXT (nam.NAM$B_TYPE)
          CALL LIB$MOVC5 (length, %VAL(nam.NAM$L_TYPE), %REF(' '),
     +                    LEN(result), %REF(result))

      ELSEIF (field_name .EQ. 'VERSION') THEN

          length = ZEXT (nam.NAM$B_VER)
          CALL LIB$MOVC5 (length, %VAL(nam.NAM$L_VER), %REF(' '),
     +                    LEN(result), %REF(result))

      ELSE      ! Return the complete file specification.

          length = ZEXT (nam.NAM$B_ESL)
          result = expanded_string(1:length)

      ENDIF


      fparse = status


      RETURN


      END
