C*******************************************************************************
C
C      STRUCHART Include File.
C
C*******************************************************************************


C...      Miscellaneous parameters.

      PARAMETER  NULL = 0                               ! Null links.


C...      Sizing parameters.

      PARAMETER  MAX_NUM_NODES = 8192                   ! Maximum number of nodes in the graph.
      PARAMETER  MAX_NUM_ROUTINES = 1024                ! Maximum number of routines.


C...      Adjacency list structure.  An adjacency list is a linked list
C      of routines that are immediately adjacent to a particular routine
C      (see the CALLS and CALLED_BY lists in the routine table below).
C      Each node in an adjacency list contains an information field and
C      a link field. The number in the information field is the index of
C      the adjacent routine in the routine table.  The link is the index
C      of the next node in the list.

      INTEGER*4         num_adj
      INTEGER*4         adj_info(MAX_NUM_NODES)
      INTEGER*4         adj_link(MAX_NUM_NODES)

      COMMON  /ADJACENCY_LISTS/  num_adj, adj_info, adj_link


C...      Routine Table.  Each routine in the table has two adjacency
C      lists attached to it. The CALLS list is a list of all routines
C      that this particular routine calls. The CALLED_BY list is a list
C      of all subroutines that this particular routine is called by.
C      The DEFINED_IN field records the source of the symbol definition,
C      i.e., where the symbol occurs in a <symbol>-calls-<reference>
C      relation.  Possible values include: 'OBJ' (object file), 'OLB'
C      (object library), 'SCI' (intermediate file), or ' ' for a symbol
C      that only appears on the right side of <x>-calls-<y> relations.

      INTEGER*4         num_routines
      CHARACTER*32      routine_name(MAX_NUM_ROUTINES)
      INTEGER*4         calls(MAX_NUM_ROUTINES)
      INTEGER*4         called_by(MAX_NUM_ROUTINES)
      CHARACTER*3       defined_in(MAX_NUM_ROUTINES)
      INTEGER*4         in_degree(MAX_NUM_ROUTINES)     ! Length of CALLED_BY list.
      LOGICAL           locked(MAX_NUM_ROUTINES)        ! Routine on current descent path?
      LOGICAL           visited(MAX_NUM_ROUTINES)       ! Routine visited during DFS print.

      COMMON  /ROUTINE_TABLE/  num_routines, routine_name,
     +                         calls, called_by, defined_in,
     +                         in_degree, locked, visited


C...      Stack used to implement recursion.

      INTEGER*4         stack_pointer
      INTEGER*4         stack(MAX_NUM_ROUTINES*2)

      COMMON  /STACK_COMMON/  stack_pointer, stack


C...      Miscellaneous Variables.  The Exclude Flag controls what
C      system names (identified by embedded "$"'s) will be ignored
C      by GET_SYMBOLS: (1) ignore all "$" routines, (2) include all
C      "$" routines, or (3) ignore a default list of "$" routines.
C      The OLB Handle is assigned and used by the LBR$ routines when
C      opening and reading a library file.  These global variables are
C      needed because of the mechanics of scanning a library file.

      INTEGER*4         exclude_flag
      PARAMETER         P_NONE = 0                      ! Include all "$".
      PARAMETER         P_DOLLARS = 1                   ! Exclude all "$".
      PARAMETER         P_DEFAULTS = 2                  ! Exclude default list.

      INTEGER*4         olb_handle                      ! Library handle.

      COMMON  /MISC_COMMON/  exclude_flag, olb_handle
